/*
	Dots CustomDataType & CustomGui Example
	(C) 2021 MAXON Computer GmbH

	Author: Sebastian Bach
	Date: 13/12/2021

	Showcases how to use custom data types in combination with custom GUIs.

  The type iSplineVisDataType is the actual data object that stores the custom data, where
	SplineVisDataType is the plugin class that represents the new data type within Cinema 4D, 
	handling the underlying iSplineVisDataType data.

  SplineVisCustomGUI is the plugin class that represents a new GUI element for that data
  type. It creates an SplineVisGUI dialog that realizes the GUI for that data type. The
  dialog includes the GeUserArea implementation SplineVisUserArea which handles the rendering of the
  custom data in a GUI.

  This example have been modified to be used a node inside the node editor.
*/
#ifndef SPLINE_VIS_H__
#define SPLINE_VIS_H__

#include "c4d_baselist.h"
#include "c4d_customdatatype.h"
#include "c4d_gui.h"
#include "c4d_symbols.h"
#include "settingsdialog.h"

#include "splinevis1.hxx"
#include "splinevis2.hxx"

#include "maxon/datatype.h"


// By me
#define KNOB_MARGIN 0.1
#define KNOB_MIN_SIZE 0.5
//#define KNOB_AN_DURATION 20.0f
#define KNOB_AN_DURATION 60.0f
#define KNOB_REFRESH_INTERVAL 20

//class SettingsDialog;
class SplineVisGUI; // forward declaration by me 

class SplineVisDataType; // forward declaration

//---------------------------
/// the actual data
//---------------------------
class iSplineVisDataType : public iCustomDataType<iSplineVisDataType>
{
	friend class SplineVisDataType;

public:
	//iSplineVisDataType() = default;
	iSplineVisDataType();
	~iSplineVisDataType();
	// explicit iSplineVisDataType(const maxon::BaseArray<Vector> &points);
	//explicit iSplineVisDataType(const BaseContainer &SplVisCon);
	explicit iSplineVisDataType(const iSplineVisDataType& cdtd);

	//---------------------------
	// The example "how to use a Classic API UI and Datatype inside the node editor" 
	// need some extra code.
	// In order to use the Classic API data type and its UI in the node editor, 
	// a copy constructor is mandatory, an assignment operator, and 
	// the DescribeIO function or the data will not be properly saved to the cinema4D's project file.
	
	//iSplineVisDataType(const iSplineVisDataType& cdtd);
	
	void operator =(const iSplineVisDataType& b)
	{
		_points.Flush();
		_points.CopyFrom(b._points) iferr_ignore("Simply ignore copy from error"_s);
		// Mein:
		 const Gradient* temp= b._gradient;
		 _gradient->CopyFrom(b._gradient);
		 //_spline->CopyFrom(b._spline);
		 _spline = b._spline;
		 _TimeScaler = b._TimeScaler;
		 //_TimeScaler.CopyFrom(b._TimeScaler);
		 //_gradient->CopyFrom(temp) iferr_ignore("Simply ignore copy from error"_s);
		
		//(*_spline).CopyFrom(b._spline) iferr_ignore("Simply ignore copy from error"_s);
		//_TimeScaler.CopyFrom(b._TimeScaler) iferr_ignore("Simply ignore copy from error"_s);
	}
		
	//MAXON_OPERATOR_EQUALITY(iSplineVisDataType, _points);
	MAXON_OPERATOR_EQUALITY(iSplineVisDataType, _points, _gradient, _spline, _TimeScaler);

	/// DescribeIO is mandatory otherwise, maxon Data cannot be saved.
	static maxon::Result<void> DescribeIO(const maxon::DataSerializeInterface& stream);
	//---------------------------
	
	// The accessors that will be used in several functions.
	maxon::BaseArray<Vector> GetValue() const;
	void SetValue(const maxon::BaseArray<Vector> &value);


	// an array of Vectors from the dot gui example
	maxon::BaseArray<Vector> _points;
	// Parameters in the SplineVis Type
	Gradient* _gradient;
	//Gradient _grad = *(Gradient::Alloc());
	//Gradient _grad;
	SplineData* _spline;
	Float _TimeScaler;
};

// For the Node API example, customDataTypeDot need to be registered as a Maxon Data.
MAXON_DATATYPE(iSplineVisDataType, "net.acestudio.datatype.splinevis");

//---------------------------
/// The user area used to display the custom datatype
//---------------------------
class SplineVisUserArea : public GeUserArea
{
public:
	SplineVisUserArea();
	virtual ~SplineVisUserArea();

	
  virtual Bool Init();
  virtual Bool InitValues();
  virtual Bool GetMinSize(Int32& w, Int32& h);

  virtual void DrawMsg(Int32 x1, Int32 y1, Int32 x2, Int32 y2, const BaseContainer& msg);
  virtual Bool InputEvent(const BaseContainer& msg);
  virtual void Timer(const BaseContainer& msg);
  //void StartAnimation();
  //void StopAnimation();

public:
  SettingsDialog _settingsDiag;

	iSplineVisDataType* _splvisData;
private:
	Float m_animationTime; // Current time in the animation
	Float m_animationDuration; // Duration of the animation
	Bool m_animating;
};

//------------
/// the iCustomGui dialog hosting the user area 
//------------
enum
{
	ID_USERAREA = 3000, ///< The ID of the user area GUI element.
};

class SplineVisGUI : public iCustomGui
{
	INSTANCEOF(SplineVisGUI, iCustomGui)

private:
	Bool _tristate;
	iSplineVisDataType _splvisData;
	SplineVisUserArea _SplineVisUserArea;

public:
	SplineVisGUI(const BaseContainer &settings, CUSTOMGUIPLUGIN *plugin);
	virtual Bool CreateLayout();
	virtual Bool InitValues();
	virtual Bool Command(Int32 id, const BaseContainer &msg);
	virtual Bool SetData(const TriState<GeData> &tristate);
	virtual TriState<GeData> GetData();
};

//---------------------
/// custom GUI data
//---------------------
static Int32 g_stringtable[] = { ID_DTYPE_SPLINE_VIS };

class SplineVisCustomGUI : public CustomGuiData
{
public:
	virtual Int32 GetId();
  virtual CDialog* Alloc(const BaseContainer& settings);
  virtual void Free(CDialog* dlg, void* userdata);
  virtual const Char* GetResourceSym();
  virtual CustomProperty* GetProperties();
  virtual Int32 GetResourceDataType(Int32*& table);

};

//---------------------------
/// The datatype class
//---------------------------
class SplineVisDataType : public CustomDataTypeClass
{
  INSTANCEOF(SplineVisDataType, CustomDataTypeClass)

public:
	//~SplineVisDataType();
	//iSplineVisDataType* _data;
	//Gradient* GetGrad();
  virtual Int32 GetId();
  virtual CustomDataType* AllocData();
  virtual void FreeData(CustomDataType* data);
  virtual Bool CopyData(const CustomDataType* src, CustomDataType* dst, AliasTrans* aliastrans);
  virtual Int32 Compare(const CustomDataType* d1, const CustomDataType* d2);
  virtual Bool WriteData(const CustomDataType* t_d, HyperFile* hf);
  virtual Bool ReadData(CustomDataType* t_d, HyperFile* hf, Int32 level);
  virtual const Char* GetResourceSym();
  virtual void GetDefaultProperties(BaseContainer& data);


  //virtual Bool GetParameter(const CustomDataType* data, const DescID& id, GeData& t_data, DESCFLAGS_GET& flags)
  //{
	 // const iSplineVisDataType* s = (iSplineVisDataType*)data;

	 // 
	 // t_data = GeData(ID_DTYPE_SPLINE_VIS, DEFAULTVALUE);
	 // flags |= DESCFLAGS_GET::PARAM_GET;
	 // 
	 // /*if (id[0].id == EXAMPLE_LONG)
	 // {
		//  t_data = GeData(s->ldata);
		//  flags |= DESCFLAGS_GET::PARAM_GET;
	 // }
	 // else if (id[0].id == EXAMPLE_STRING)
	 // {
		//  t_data = GeData(s->sdata);
		//  flags |= DESCFLAGS_GET::PARAM_GET;
	 // }
	 // else if (id[0].id == EXAMPLE_VECTOR)
	 // {
		//  HandleDescGetVector(id, s->vdata, t_data, flags);
	 // }*/


	 // return SUPER::GetParameter(data, id, t_data, flags);
  //}





};

//---------------------------
/// Register
//---------------------------
Bool RegisterSplineVisDatatypeCustomGUI();


#endif // SPLINE_VIS_H__
